(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/demo-server.js                                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var DEMO_EXPIRATION_MS = 60 * 60 * 1000;                               // 17
var DEMO_GRACE_MS = 10 * 60 * 1000; // time between expiration and deletion
                                                                       //
Accounts.validateLoginAttempt(function (attempt) {                     // 20
  // Enforce expiration times for demo accounts.                       //
                                                                       //
  if (attempt.user && attempt.user.expires) {                          // 23
    (function () {                                                     //
      var expireIn = attempt.user.expires.getTime() - Date.now() + DEMO_GRACE_MS;
      if (expireIn < 0) {                                              // 25
        throw new Meteor.Error(403, "This demo account has expired.");
      }                                                                //
                                                                       //
      if (!allowDemo) {                                                // 29
        // An expiration time implies that this is a demo account, but those have been disabled.
        // Perhaps the config was just changed.                        //
        throw new Meteor.Error(403, "Demo accounts have been disabled.");
      }                                                                //
                                                                       //
      // Force connection close when account expires, so that the client reconnects and
      // re-authenticates, which fails.                                //
      var connection = attempt.connection;                             // 37
      var handle = Meteor.setTimeout(function () {                     // 38
        connection.close();                                            // 38
      }, expireIn);                                                    //
                                                                       //
      connection.onClose(function () {                                 // 40
        Meteor.clearTimeout(handle);                                   // 40
      });                                                              //
    })();                                                              //
  }                                                                    //
                                                                       //
  return true;                                                         // 43
});                                                                    //
                                                                       //
function cleanupExpiredUsers() {                                       // 46
  // Delete expired demo accounts and all their grains.                //
                                                                       //
  var now = new Date(Date.now() - DEMO_GRACE_MS);                      // 49
  Meteor.users.find({ expires: { $lt: now } }, { fields: { _id: 1, loginIdentities: 1, lastActive: 1, appDemoId: 1 } }).forEach(function (user) {
    Grains.find({ userId: user._id }, { fields: { _id: 1, lastUsed: 1, appId: 1 } }).forEach(function (grain) {
      console.log("delete grain: " + grain._id);                       // 55
      globalDb.removeApiTokens({ grainId: grain._id });                // 56
      Grains.remove(grain._id);                                        // 57
      if (grain.lastUsed) {                                            // 58
        DeleteStats.insert({ type: "demoGrain", lastActive: grain.lastUsed, appId: grain.appId });
      }                                                                //
                                                                       //
      globalBackend.deleteGrain(grain._id, user._id);                  // 62
    });                                                                //
                                                                       //
    console.log("delete user: " + user._id);                           // 65
    // We intentionally do not do `ApiTokens.remove({accountId: user._id})`, because some such
    // tokens might still play an active role in the sharing graph.    //
    Contacts.remove({ ownerId: user._id });                            // 68
    UserActions.remove({ userId: user._id });                          // 69
    Notifications.remove({ userId: user._id });                        // 70
    Meteor.users.remove(user._id);                                     // 71
    waitPromise(globalBackend.cap().deleteUser(user._id));             // 72
    if (user.loginIdentities && user.lastActive) {                     // 73
      // When deleting a user, we can specify it as a "normal" user    //
      // (type: user) or as a user who started out by using the app    //
      // demo feature (type: appDemoUser).                             //
      var deleteStatsType = "demoUser";                                // 77
      var isAppDemoUser = !!user.appDemoId;                            // 78
      if (isAppDemoUser) {                                             // 79
        deleteStatsType = "appDemoUser";                               // 80
      }                                                                //
                                                                       //
      // Intentionally record deleted users at time of deletion to avoid miscounting users that
      // were demoing just before the day rolled over.                 //
      DeleteStats.insert({ type: deleteStatsType, lastActive: new Date(), appId: user.appDemoId });
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
if (allowDemo) {                                                       // 90
  Meteor.methods({                                                     // 91
    createDemoUser: function (displayName, appDemoId) {                // 92
      // This is a login method that creates a new temporary user      //
      // every time it is used.                                        //
      //                                                               //
      // appDemoId is important for stats; see cleanupExpiredUsers().  //
      check(displayName, String);                                      // 97
      check(appDemoId, Match.OneOf(undefined, null, String));          // 98
                                                                       //
      // Create the new user.                                          //
      var newUser = { expires: new Date(Date.now() + DEMO_EXPIRATION_MS) };
      if (appDemoId) {                                                 // 102
        newUser.appDemoId = appDemoId;                                 // 103
      }                                                                //
                                                                       //
      var userId = Accounts.insertUserDoc({ profile: { name: displayName } }, newUser);
                                                                       //
      // Log them in on this connection.                               //
      return Accounts._loginMethod(this, "createDemoUser", arguments, "demo", function () {
        return { userId: userId };                                     // 110
      });                                                              //
    },                                                                 //
                                                                       //
    testExpireDemo: function () {                                      // 113
      if (!isDemoUser()) throw new Meteor.Error(403, "not a demo user");
                                                                       //
      var newExpires = new Date(Date.now() + 15000);                   // 116
      if (Meteor.user().expires.getTime() < newExpires.getTime()) {    // 117
        throw new Meteor.Error(403, "can't exend demo");               // 118
      }                                                                //
                                                                       //
      Meteor.users.update(this.userId, { $set: { expires: newExpires } });
    }                                                                  //
  });                                                                  //
                                                                       //
  // If demo mode is enabled, we permit the client to subscribe to     //
  // information about an app by appId. If this were available in      //
  // non-demo mode, then anonymous users could effectively ask the     //
  // server which apps are installed.                                  //
  Meteor.publish("appInfo", function (appId) {                         // 129
    // This publishes info about an app, including the latest          //
    // version of it. Once you log in, it also publishes your          //
    // list of UserActions.                                            //
    check(appId, String);                                              // 133
                                                                       //
    var packageCursor = Packages.find({ appId: appId }, { sort: { "manifest.appVersion": -1 } });
                                                                       //
    var pkg = packageCursor.fetch()[0];                                // 139
                                                                       //
    // This allows us to avoid creating duplicate UserActions.         //
    if (this.userId) {                                                 // 142
      return [packageCursor, UserActions.find({ userId: this.userId, appId: appId })];
    }                                                                  //
                                                                       //
    return packageCursor;                                              // 149
  });                                                                  //
                                                                       //
  SandstormDb.periodicCleanup(DEMO_EXPIRATION_MS, cleanupExpiredUsers);
} else {                                                               //
  // Just run once, in case the config just changed from allowing demos to prohibiting them.
  Meteor.setTimeout(cleanupExpiredUsers, DEMO_EXPIRATION_MS + DEMO_GRACE_MS);
}                                                                      //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=demo-server.js.map
